"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WASM_IMPORT_STATEMENT_REX = void 0;
const vite_1 = require("vite");
const pluginutils_1 = require("@rollup/pluginutils");
const fs_1 = require("fs");
const parser_1 = require("./parser");
const path_1 = require("path");
const DEFAULT_OPTIONS = {
    include: [],
    exclude: [],
};
// matching like: 'import wasm from "xxx.wasm"'
exports.WASM_IMPORT_STATEMENT_REX = /import\s*(\S*)\s*from\s*['"](.*\.wasm).*/;
exports.default = (options = {}) => {
    // init options
    options = Object.assign(DEFAULT_OPTIONS, options);
    // normalize path of include and exclude. Then create a filter.
    const include = options.include.map(item => (0, vite_1.normalizePath)(item));
    const exclude = options.exclude.map(item => (0, vite_1.normalizePath)(item));
    const filter = (0, pluginutils_1.createFilter)(include, exclude);
    return {
        name: 'vite:load-wasm',
        enforce: 'pre',
        async transform(code, id) {
            // check file id
            if (!filter(id)) {
                return null;
            }
            // regular matching and check
            const rexRes = exports.WASM_IMPORT_STATEMENT_REX.exec(code);
            if (rexRes === null) {
                return null;
            }
            if (!rexRes[0] || !rexRes[1] || !rexRes[2]) {
                console.log('vite-plugin-load-wasm: regular matching result fail');
                return null;
            }
            // init matchStr, importName, wasmFileName
            const [matchStr, importName, wasmFileName] = [
                rexRes[0],
                rexRes[1],
                rexRes[2],
            ];
            // get importObj
            const info = (0, parser_1.getImportObjInfo)((0, fs_1.readFileSync)((0, path_1.join)((0, path_1.dirname)(id), wasmFileName)));
            if (info.size > 1) {
                console.log('vite-plugin-load-wasm: import item number should less than 1');
                return null;
            }
            let importFileName;
            for (let name of info.keys()) {
                importFileName = name;
                break;
            }
            // replace string
            let replaceStr;
            if (!!importFileName) {
                replaceStr = `import init from '${wasmFileName}'
let imports = {}
imports['${importFileName}'] = {${info.get(importFileName).join(', ')}}
const ${importName} = await init(imports)
`;
            }
            else {
                replaceStr = `import init from '${wasmFileName}'
const ${importName} = await init()
`;
            }
            // console.log(code.replace(matchStr, replaceStr));
            return code.replace(matchStr, replaceStr);
        },
    };
};
